<!--
   Component
   sso picker from quaero
   Events :
      - ssopicked triggered on select item
-->
<w-quaero>
   <!-- layout -->
   
   <input
      ref="quaero"
      type="text"
      list="quaero-data"
      autocomplete="off"
      placeholder="Corps du système solaire"/>
   <ul id="quaero-data" show={ hasQuaeroItems }>
      <li each={ item, i in quaeroItems } data-id={ i }><w-quaero-item data={ item }/></li>
   </ul>
         
   <!-- style -->
   
   <style scoped>
      :scope {display:block;border:thin dotted red;}
      input {
         display:inline-block;
         height:40px;
         width:100%;
         vertical-align:top;
         background-color:inherit;
         font-size:1em;
         color:black;
         background:url('./img/ic_search_black_24px.svg') no-repeat right 16px center;
         padding-left:16px;
         border:none;
      }
      li {
         list-style:none;
         min-height:40px;
         line-height:40px;
         padding-left:16px;
      }
      .hover {background-color:green;}
      .border {border-bottom:thin solid black;}
   </style>
   
   <!-- script -->
   
   <script>
      'use strict';
      
      this.quaeroItems = [];
      this.hasQuaeroItems = this.quaeroItems.length;
      this.selectedItem = null;
      
      select(elt) {
         var idx = Number.parseInt(elt.getAttribute('data-id'));
         var sso = this.quaeroItems[idx].sso;
         this.trigger('ssopicked', sso);
         window.console.log(sso);
      }
      
      this.on('mount', function() {
         var self = this;
         
         /* input handler on search box (fill) */
         
         this.refs.quaero.addEventListener('input', function(e) {
            window.console.log(e);
            
            /* empty quaero data and reset selection if any */
            
            if(self.quaeroItems.length) {
               self.quaeroItems.splice(0);
            }
            self.hasQuaeroItems = self.quaeroItems.length;
            self.selectedItem = null;
            
            var term = this.value.trim();
            
            if(term.length > 0) {
               ssodnet.quaero.autocomplete(term).then(
                  function(response) {                     
                     var r = JSON.parse(response);
                     var data = r.data;
                     var re = new RegExp(term, 'ig');
                     
                     for(var i = 0, l = data.length; i < l;i++) {
                        /*
                           'parent' property of sso means something in riot.js so it is not possible to just call
                           self.quaeroItems.push(data[i]);
                        */
                        
                        self.quaeroItems.push({sso: data[i], re: re});
                     }
   
                     self.hasQuaeroItems = self.quaeroItems.length;
                     self.update();
                  },
                  function(error) {
                     window.console.error(error);
                  }
               );
            }
            
            self.update();
         });
         
         /* keyup handler on search box (handle keyboard navigation and selection) */
         
         this.refs.quaero.addEventListener('keyup', function(e) {
            window.console.log(e);
            
            if(self.hasQuaeroItems) {
               /* KeyboardEvent.key cf. caniuse or use 'which' property (riotjs) ? */
               
               switch(e.key) {
                  case 'ArrowUp':
                     if(!self.selectedItem) {
                        self.selectedItem = self.root.querySelector('#quaero-data > li:last-child');
                     } else {
                        self.selectedItem.classList.remove('hover');
                        self.selectedItem = self.selectedItem.previousElementSibling || self.root.querySelector('#quaero-data > li:last-child');
                     }
                     self.selectedItem.classList.add('hover');
                     break;
                  case 'ArrowDown':
                     if(!self.selectedItem) {
                        self.selectedItem = self.root.querySelector('#quaero-data > li:first-child');
                     } else {
                        self.selectedItem.classList.remove('hover');
                        self.selectedItem = self.selectedItem.nextElementSibling || self.root.querySelector('#quaero-data > li:first-child');
                     }
                     self.selectedItem.classList.add('hover');
                     break;
                  case 'Enter':
                     if(self.selectedItem) {
                        self.select(self.selectedItem);
                     }
                     break;
                  default:
                     break;
               }
            }
         });
         
         /* mouse handler on search results list (mouse selection and navigation)
            only <li> can handle these events inside #quaero-data because
            <w-quaero-item> can not handle mouse events (by css design).
          */
         
         var quaeroData = this.root.querySelector('#quaero-data');
         
         quaeroData.addEventListener('click', function(e) {
            window.console.log(e);
            if(e.target && e.target.nodeName == 'LI') {
               self.select(e.target);
            }
         });
         
         quaeroData.addEventListener('mouseover', function(e) {
            window.console.log(e);
            if(e.target && e.target.nodeName == 'LI') {
               window.console.log(e);               
               if(self.selectedItem) {
                  self.selectedItem.classList.remove('hover');
               }
               self.selectedItem = e.target;
               self.selectedItem.classList.add('hover');
            }
         });
      })
      
      this.on('update', function() {
         (this.hasQuaeroItems) ? this.refs.quaero.classList.add('border') : this.refs.quaero.classList.remove('border');
      })
   </script>
</w-quaero>
      
<w-quaero-item>
   <style scoped>
      :scope {pointer-events:none;}
      em {font-weight:bolder;font-style:normal;}
      span {font-weight:lighter;font-size:smaller;}
   </style>
   <script>
      'use strict';
      
      label(sso, re) {
         /* highlighted name */
         
         var hname = sso.name.replace(re, '<em>$&</em>');
         
         /* highlighted aliases */
         
         var haliases = [];
         if(sso.hasOwnProperty('aliases')) {
            for(var i = 0, l = sso.aliases.length; i < l; i++) {
               var halias = sso.aliases[i].replace(re, '<em>$&</em>');
               if(halias != sso.aliases[i]) {
                  haliases.push(halias);
               }
            }
         }
         
         /* format label */
         
         var label = (hname != sso.name) ? hname : sso.name;
         
         if(haliases.length) {
            label = label + ' (' + haliases.join(', ') + ')';
         }
         
         return label + ' <span>' + sso.type + '</span>';
      }
      
      this.root.innerHTML = this.label(opts.data.sso, opts.data.re);
   </script>
</w-quaero-item>